<?php
/**
 * Copyright (с) Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2023 All Rights Reserved
 */

namespace WP_Rocket\Engine\AccelerateWp;

use Firebase\JWT\JWT;
use stdClass;
use WP_Error;

/**
 * Abstract class for the AccelerateWP SAAS API service.
 *
 * Handles common types of request while adding the correct authentication headers.
 *
 * @since 3.12.6.1_1.1-1
 */
abstract class AbstractAPIClient {

	const AUTH_HEADER_NAME = 'AccessKey';

	/**
	 * JWT token for requests
	 *
	 * @var string
	 */
	private $jwt = null;

	/**
	 * Puts together a value of the authentication headers.
	 *
	 * @since 3.12.6.1_1.1-1
	 *
	 * @return string|WP_Error Authentication header value. WP_Error if there is a problem with API configuration.
	 */
	protected function get_auth_header() {

		// Check that we have non-empty JWT token.
		$jwt = $this->get_jwt();
		if ( ! is_string( $jwt ) || 0 === strlen( $jwt ) ) {
			return $this->build_error_for_missing_signature_file();
		}

		// Check that we have non-empty unique ID.
		$unique_id = $this->get_awp_unique_id();
		if ( ! is_string( $unique_id ) || 0 === strlen( $unique_id ) ) {
			return $this->build_error_for_missing_unique_id();
		}

		return $jwt . '.' . $unique_id;
	}

	/**
	 * Makes a remote GET request to given endpoint. It automatically prepends the API base URL and adds authentication
	 * headers.
	 *
	 * @param string $endpoint_url Relative endpoint URL.
	 *
	 * @return array|\WP_Error
	 */
	protected function remote_get( $endpoint_url ) {

		$auth_header = $this->get_auth_header();
		if ( is_wp_error( $auth_header ) ) {
			return $auth_header;
		}

		return wp_remote_get(
			trailingslashit( $this->get_saas_url() ) . $endpoint_url,
			[
				'sslverify' => false,
				'timeout'   => $this->get_max_timeout(),
				'headers'   => [
					self::AUTH_HEADER_NAME => $auth_header,
					'Accept'               => 'application/json',
				],
			]
		);
	}

	/**
	 * Makes a remote POST request to given endpoint. It automatically prepends the API base URL, adds authentication
	 * headers and include the parameters in the request body.
	 *
	 * @param string $endpoint_url Relative endpoint URL.
	 * @param array  $params       Optional. Parameters needed to be sent in the body. Default: [].
	 *
	 * @return array|\WP_Error
	 */
	protected function remote_post( $endpoint_url, $params ) {

		$auth_header = $this->get_auth_header();
		if ( is_wp_error( $auth_header ) ) {
			return $auth_header;
		}

		return wp_remote_post(
			trailingslashit( $this->get_saas_url() ) . $endpoint_url,
			[
				'body'      => $params,
				'sslverify' => false,
				'timeout'   => $this->get_max_timeout(),
				'headers'   => [
					self::AUTH_HEADER_NAME => $auth_header,
					'Accept'               => 'application/json',
				],
			]
		);
	}

	/**
	 * Get the status of response.
	 *
	 * @since 3.6
	 *
	 * @param int      $response_code Response code to check success or failure.
	 * @param stdClass $response_data Object of data returned from request.
	 *
	 * @return bool success or failed.
	 */
	protected function get_response_success( $response_code, $response_data ) {
		return (
			200 === $response_code
			&&
			! empty( $response_data )
			&&
			(
				(
					isset( $response_data->status )
					&&
					200 === $response_data->status
				)
				||
				(
					isset( $response_data->data )
					&&
					isset( $response_data->data->id )
				)
			)
		);
	}

	/**
	 * Get response status code/number.
	 *
	 * @since 3.6
	 *
	 * @param array|WP_Error $response The response or WP_Error on failure.
	 * @param null|int       $status   Optional. Status code to overwrite the response status. Default: null.
	 *
	 * @return int status code|number of response.
	 */
	protected function get_response_status( $response, $status = null ) {
		if ( ! is_null( $status ) ) {
			return (int) $status;
		}

		return (int) wp_remote_retrieve_response_code( $response );
	}

	/**
	 * Get response data from the API.
	 *
	 * @since 3.6
	 *
	 * @param array|WP_Error $response The response or WP_Error on failure.
	 *
	 * @return mixed response of API.
	 */
	protected function get_response_data( $response ) {
		return json_decode( wp_remote_retrieve_body( $response ) );
	}

	/**
	 * Generate JWT.
	 *
	 * @return string
	 */
	public function get_jwt() {
		if ( is_null( $this->jwt ) ) {
			$signature_data = $this->get_jwt_secret();

			if ( ! is_wp_error( $signature_data ) ) {
				$unique_id = $this->get_awp_unique_id();
				$payload   = [
					'iss' => defined( 'WP_HOME' ) ? WP_HOME : (string) get_option( 'home' ),
					'sub' => $unique_id,
					'exp' => time() + 1200, // 20 min
				];

				$this->jwt = JWT::encode(
					$payload,
					$signature_data['key'] . $unique_id,
					'HS256'
				);
			} else {
				$this->jwt = null;
			}
		}

		return $this->jwt;
	}

	/**
	 * Get JWT secret from ini file
	 *
	 * @return array|WP_Error
	 */
	public function get_jwt_secret() {
		if ( $this->is_signature_file_readable() ) {
			$signature_data = parse_ini_file( $this->get_signature_file_path(), false );
		} else {
			return Sentry::enrich_wp_error( new WP_Error( 500, esc_html__( 'Cannot parse sign.ini file', 'rocket' ) ) );
		}

		return $signature_data;
	}

	/**
	 * Get saas url.
	 *
	 * @since 3.12.6.1_1.1-2 Updated to automatically detect staging mode.
	 *
	 * @return string
	 */
	public function get_saas_url() {
		$url = 'https://awp-saas.cloudlinux.com/';

		if ( @file_exists( '/opt/cloudlinux/staging_mode' ) ) { // phpcs:ignore WordPress.PHP.NoSilencedErrors.Discouraged
			$url = 'https://awp-saas.staging.cloudlinux.com/';
		}

		if ( defined( 'AWP_SAAS_URL' ) ) {
			$url = AWP_SAAS_URL;
		}

		return trailingslashit( $url );
	}

	/**
	 * Retrieves the AccelerateWP unique ID to be used for authenticating against the API.
	 *
	 * @since 3.12.6.1_1.1-1
	 *
	 * @return string
	 */
	abstract protected function get_awp_unique_id();

	/**
	 * Builds the timeout limit for queries talking with the SaaS service.
	 *
	 * Based on local php max_execution_time in php.ini
	 *
	 * @since 3.12.6.1_1.1-1
	 * @return int
	 **/
	protected function get_max_timeout() {
		$timeout = (int) ini_get( 'max_execution_time' );

		// Ensure exec time set in php.ini.
		if ( ! $timeout ) {
			$timeout = 30;
		}

		return (int) ceil( $timeout * 0.8 );
	}

	/**
	 * Get the path to the signature file.
	 *
	 * @return string
	 */
	private function get_signature_file_path() {
		return rocket_get_constant( 'WP_ROCKET_UPDATE_PATH' ) . DIRECTORY_SEPARATOR . 'sign.ini';
	}

	/**
	 * Checks if the signature file is readable.
	 *
	 * @return bool
	 */
	private function is_signature_file_readable() {
		return is_readable( $this->get_signature_file_path() );
	}

	/**
	 * Validates the API configuration and generates a list of errors if any.
	 *
	 * @return WP_Error[] List of errors.
	 */
	public function get_configuration_errors() {

		$result = [];

		// Check if the signature file is readable.
		if ( ! $this->is_signature_file_readable() ) {
			$result[] = $this->build_error_for_missing_signature_file();
		}

		// Check if the unique ID is set.
		if ( 0 === strlen( $this->get_awp_unique_id() ) ) {
			$result[] = $this->build_error_for_missing_unique_id();
		}

		return $result;
	}

	/**
	 * Builds an error for a missing signature file.
	 *
	 * @return WP_Error
	 *
	 * @since 3.12.6.1_1.1-1
	 */
	private function build_error_for_missing_signature_file() {
		return Sentry::enrich_wp_error(
			new WP_Error(
				'apw_sass_api_jwt_configuration',
				sprintf(
					// translators: %s = path to the signature file.
					esc_html__( 'JWT signature file is missing: %s', 'rocket' ),
					'<code>' . $this->get_signature_file_path() . '</code>'
				)
			)
		);
	}

	/**
	 * Builds an error for a missing unique ID.
	 *
	 * @return WP_Error
	 *
	 * @since 3.12.6.1_1.1-1
	 */
	private function build_error_for_missing_unique_id() {
		return Sentry::enrich_wp_error(
			new WP_Error(
				'apw_sass_api_unique_id_missing',
				esc_html__( 'Unique ID is missing', 'rocket' )
			)
		);
	}
}
